@@ -35,6 +35,43 @@ namespace Aws
             class MqttClient;
             class MqttConnection;
 
+            /**
+             * The data returned when the connection closed callback is invoked in a connection.
+             * Note: This class is currently empty, but this may contain data in the future.
+             */
+            struct OnConnectionClosedData
+            {
+            };
+
+            /**
+             * The data returned when the connection success callback is invoked in a connection.
+             */
+            struct OnConnectionSuccessData
+            {
+                /**
+                 * The Connect return code received from the server.
+                 */
+                ReturnCode returnCode;
+
+                /**
+                 * Returns whether a session was present and resumed for this successful connection.
+                 * Will be set to true if the connection resumed an already present MQTT connection session.
+                 */
+                bool sessionPresent;
+            };
+
+            /**
+             * The data returned when the connection failure callback is invoked in a connection.
+             */
+            struct OnConnectionFailureData
+            {
+                /**
+                 * The AWS CRT error code for the connection failure.
+                 * Use Aws::Crt::ErrorDebugString to get a human readable string from the error code.
+                 */
+                int error;
+            };
+
             /**
              * Invoked Upon Connection loss.
              */
@@ -52,6 +89,31 @@ namespace Aws
             using OnConnectionCompletedHandler = std::function<
                 void(MqttConnection &connection, int errorCode, ReturnCode returnCode, bool sessionPresent)>;
 
+            /**
+             * Invoked when a connection is disconnected and shutdown successfully.
+             *
+             * Note: Currently callbackData will always be nullptr, but this may change in the future to send additional
+             * data.
+             */
+            using OnConnectionClosedHandler =
+                std::function<void(MqttConnection &connection, OnConnectionClosedData *callbackData)>;
+
+            /**
+             * Invoked whenever the connection successfully connects.
+             *
+             * This callback is invoked for every successful connect and every successful reconnect.
+             */
+            using OnConnectionSuccessHandler =
+                std::function<void(MqttConnection &connection, OnConnectionSuccessData *callbackData)>;
+
+            /**
+             * Invoked whenever the connection fails to connect.
+             *
+             * This callback is invoked for every failed connect and every failed reconnect.
+             */
+            using OnConnectionFailureHandler =
+                std::function<void(MqttConnection &connection, OnConnectionFailureData *callbackData)>;
+
             /**
              * Invoked when a suback message is received.
              */
@@ -75,13 +137,13 @@ namespace Aws
 
             /**
              * Invoked upon receipt of a Publish message on a subscribed topic.
-             * @param connection    The connection object
-             * @param topic         The information channel to which the payload data was published.
-             * @param payload       The payload data.
-             * @param dup           DUP flag. If true, this might be re-delivery of an earlier
+             * - connection:    The connection object
+             * - topic:         The information channel to which the payload data was published.
+             * - payload:       The payload data.
+             * - dup:           DUP flag. If true, this might be re-delivery of an earlier
              *                      attempt to send the message.
-             * @param qos           Quality of Service used to deliver the message.
-             * @param retain        Retain flag. If true, the message was sent as a result of
+             * - qos:           Quality of Service used to deliver the message.
+             * - retain:        Retain flag. If true, the message was sent as a result of
              *                      a new subscription being made by the client.
              */
             using OnMessageReceivedHandler = std::function<void(
@@ -121,6 +183,34 @@ namespace Aws
             using OnWebSocketHandshakeIntercept = std::function<
                 void(std::shared_ptr<Http::HttpRequest> req, const OnWebSocketHandshakeInterceptComplete &onComplete)>;
 
+            /* Simple statistics about the current state of the client's queue of operations */
+            struct AWS_CRT_CPP_API MqttConnectionOperationStatistics
+            {
+                /*
+                 * total number of operations submitted to the connection that have not yet been completed.  Unacked
+                 * operations are a subset of this.
+                 */
+                uint64_t incompleteOperationCount;
+
+                /*
+                 * total packet size of operations submitted to the connection that have not yet been completed. Unacked
+                 * operations are a subset of this.
+                 */
+                uint64_t incompleteOperationSize;
+
+                /*
+                 * total number of operations that have been sent to the server and are waiting for a corresponding ACK
+                 * before they can be completed.
+                 */
+                uint64_t unackedOperationCount;
+
+                /*
+                 * total packet size of operations that have been sent to the server and are waiting for a corresponding
+                 * ACK before they can be completed.
+                 */
+                uint64_t unackedOperationSize;
+            };
+
             /**
              * Represents a persistent Mqtt Connection. The memory is owned by MqttClient.
              * To get a new instance of this class, see MqttClient::NewConnection. Unless
@@ -325,11 +415,21 @@ namespace Aws
                     const ByteBuf &payload,
                     OnOperationCompleteHandler &&onOpComplete) noexcept;
 
+                /**
+                 * Get the statistics about the current state of the connection's queue of operations
+                 *
+                 * @return MqttConnectionOperationStatistics
+                 */
+                const MqttConnectionOperationStatistics &GetOperationStatistics() noexcept;
+
                 OnConnectionInterruptedHandler OnConnectionInterrupted;
                 OnConnectionResumedHandler OnConnectionResumed;
                 OnConnectionCompletedHandler OnConnectionCompleted;
                 OnDisconnectHandler OnDisconnect;
                 OnWebSocketHandshakeIntercept WebsocketInterceptor;
+                OnConnectionClosedHandler OnConnectionClosed;
+                OnConnectionSuccessHandler OnConnectionSuccess;
+                OnConnectionFailureHandler OnConnectionFailure;
 
               private:
                 aws_mqtt_client *m_owningClient;
@@ -343,6 +443,7 @@ namespace Aws
                 void *m_onAnyCbData;
                 bool m_useTls;
                 bool m_useWebsocket;
+                MqttConnectionOperationStatistics m_operationStatistics;
 
                 MqttConnection(
                     aws_mqtt_client *client,
@@ -366,12 +467,26 @@ namespace Aws
                     enum aws_mqtt_connect_return_code returnCode,
                     bool sessionPresent,
                     void *userData);
+
+                static void s_onConnectionSuccess(
+                    aws_mqtt_client_connection *,
+                    ReturnCode returnCode,
+                    bool sessionPresent,
+                    void *userData);
+
+                static void s_onConnectionFailure(aws_mqtt_client_connection *, int errorCode, void *userData);
+
                 static void s_onConnectionResumed(
                     aws_mqtt_client_connection *,
                     ReturnCode returnCode,
                     bool sessionPresent,
                     void *userData);
 
+                static void s_onConnectionClosed(
+                    aws_mqtt_client_connection *,
+                    on_connection_closed_data *data,
+                    void *userData);
+
                 static void s_onDisconnect(aws_mqtt_client_connection *connection, void *userData);
                 static void s_onPublish(
                     aws_mqtt_client_connection *connection,
@@ -425,7 +540,15 @@ namespace Aws
                 /**
                  * Initialize an MqttClient using bootstrap and allocator
                  */
-                MqttClient(Io::ClientBootstrap &bootstrap, Allocator *allocator = g_allocator) noexcept;
+                MqttClient(Io::ClientBootstrap &bootstrap, Allocator *allocator = ApiAllocator()) noexcept;
+
+                /**
+                 * Initialize an MqttClient using a allocator and the default ClientBootstrap
+                 *
+                 * For more information on the default ClientBootstrap see
+                 * Aws::Crt::ApiHandle::GetOrCreateStaticDefaultClientBootstrap
+                 */
+                MqttClient(Allocator *allocator = ApiAllocator()) noexcept;
 
                 ~MqttClient();
                 MqttClient(const MqttClient &) = delete;
